> ![MikroE](http://www.mikroe.com/img/designs/beta/logo_small.png)
> #M-BUS RF click#
> ##By [MikroElektronika](http://www.mikroe.com)
---

## Installation
Use the package manager to install the library for your architecture.

###Example on PIC32
```
#include <stdint.h>
#include <stdbool.h>
#include <stddef.h>
#include "mbus_hw.h"

#define BUFFER_SIZE 300

sbit MBUS_RST     at RC1_bit;
sbit MBUS_RST_DIR at TRISC1_bit;
sbit MBUS_RTS     at RC2_bit;
sbit MBUS_RTS_DIR at TRISC2_bit;
sbit MBUS_CTS     at RE8_bit;
sbit MBUS_CTS_DIR at TRISE8_bit;

bool response_finished;
bool ok_response;
bool my_receive_flag = false;

void check_status( status_t status )
{
    if ( status == OK_STATUS )
        UART5_Write_Text( "OK Status\r\n" );
    else
        UART5_Write_Text( "Error Occurred\r\n" );
}

void system_setup( void )
{
    //GPIOs
    MBUS_CTS_DIR = 0;
    MBUS_RST_DIR = 0;
    MBUS_RTS_DIR = 1;
    
    //UART for terminal
    UART5_Init( 9600 );
    Delay_ms(300);

    //UART for MBUS
    UART2_Init( 19200 );
    Delay_ms(1000);

    UART5_Write_Text("*******************************************\r\n" );
    UART5_Write_Text("************* Setup Transmitter ***********\r\n" );
    UART5_Write_Text("              Uart initialized             \r\n");
    UART5_Write_Text("                 Uarts ready               \r\n" );
    UART5_Write_Text("            Starting Transmission          \r\n" );
    UART5_Write_Text("*******************************************\r\n" );
    UART5_Write_Text("*******************************************\r\n" );
    mbus_init();                                                /**< Initialize MBus HAL Layer */
    UART5_Write_Text("**** MBus Click Test Initialized." );
    Delay_ms(500);
    UART5_Write_Text( "." );
    Delay_ms(500);
    UART5_Write_Text( "." );
    Delay_ms(500);
    UART5_Write_Text( "    ****\r\n" );

    //UART Interrupt Setup
    AD1PCFG = 0xFFFF;

    U2IP0_bit = 0;
    U2IP1_bit = 1;
    U2IP2_bit = 1;
    U2RXIE_bit = 1;
    EnableInterrupts();
}

void clear_transmit_buffer( transmit_frame_t *transmit_frame )
{
    uint8_t i = 0;

    for ( i = 0; i < 100; i++ )
        transmit_frame->data_field[i] = 0;
}

void UART2interrupt() iv IVT_UART_2 ilevel 6 ics ICS_AUTO
{
      if( IFS1 & ( 1 << U2RXIF ))
      {
           char tmp = UART2_Read();
           mbus_rx_isr( tmp );
           //UART5_Write( tmp );
           U2RXIF_bit = 0;
      }
}

```

```
void main()
{
    // Local Declarations
    char long_text[100] = { 0 };
    uint16_t my_threshold = 0;
    uint8_t status = 0;
    uint8_t buffer[20] = { 0 };
    uint8_t new_meter[6] = { 0, 1, 2, 3, 4, 5 };
    uint8_t received_meter[8] = { 0 };
    date_time_t *my_time;
    transmit_frame_t *transmit_frame;
    status_t my_status = OK_STATUS;
    uint8_t serial_num_buff[20] = { 0 };

    // Malloc space for the Transmit Frame
    MM_Init();
    transmit_frame = Malloc( sizeof( transmit_frame_t ) );
    my_time = Malloc( sizeof( date_time_t ) );
    // Setup GPIOs, communication pins, interrupts, e.t.c.
    system_setup();


    //Initialize Conifguration Mode
    UART5_Write_Text( "\r\n\r\n\r\n" );
    UART5_Write_Text( "Entering Config Mode\r\n" );
    my_status = mbus_at_init();
    check_status( my_status );
    
    // Get firmware bootloader
    UART5_Write_Text( "Getting Firmware Version\r\n" );
    my_status = mbus_get_firmware_bootloader_vers( buffer );
    Delay_ms(200);
    check_status( my_status );
    UART5_Write_Text( buffer );
    UART5_Write_Text( "\r\n" );

    // Get single channel lbt delay
    UART5_Write_Text( "LBT Delay: " );
    my_status = mbus_get_single_channel_lbt_delay( &my_threshold );
    Delay_ms(200);
    check_status( my_status );
    LongToStr( my_threshold, buffer );
    UART5_Write_Text( buffer );
    UART5_Write_Text( "\r\n" );

    // Set Date / Time
    my_time->month  = 7;
    my_time->day    = 21;
    my_time->hour   = 14;
    my_time->minute = 30;
    my_time->year   = 16;
    my_time->second = 27;
    UART5_Write_Text( "Setting Date / Time\r\n" );
    check_status( mbus_set_date_time( my_time ) );

    // Get Date / Time
    UART5_Write_Text( "Getting Date / Time\r\n" );
    Delay_ms(200);
    status = mbus_get_date_time( my_time );
    UART5_Write_Text( "Got past Get Date\r\n" );
    if ( status == OK_STATUS )
    {
        UART5_Write_Text( "Month: " );           //Month
        IntToStr( my_time->month, buffer );
        UART5_Write_Text( buffer );
        UART5_Write_Text( "\r\n" );
        UART5_Write_Text( "Hour: " );            //Hour
        IntToStr( my_time->hour, buffer );
        UART5_Write_Text( buffer );
        UART5_Write_Text( "\r\n" );
        UART5_Write_Text( "Year: " );            //Year
        IntToStr( my_time->year, buffer );
        UART5_Write_Text( buffer );
        UART5_Write_Text( "\r\n" );
        UART5_Write_Text( "Day: " );             //Day
        IntToStr( my_time->day, buffer );
        UART5_Write_Text( buffer );
        UART5_Write_Text( "\r\n" );
        UART5_Write_Text( "Minute: " );          //Minute
        IntToStr( my_time->minute, buffer );
        UART5_Write_Text( buffer );
        UART5_Write_Text( "\r\n" );
        UART5_Write_Text( "Second: " );          //Second
        IntToStr( my_time->second, buffer );
        UART5_Write_Text( buffer );
        UART5_Write_Text( "\r\n" );
    }
    else
        UART5_Write_Text( "Not Successful getting of the date\r\n" );

    // Get Serial Number
    UART5_Write_Text( "Getting Serial Number\r\n" );
    mbus_get_serial_number( serial_num_buff );
    UART5_Write_Text( serial_num_buff );
    UART5_Write_Text( "\r\n" );

    // Get RX / TX Pins
    UART5_Write_Text( "Getting RX/TX Pins\r\n" );
    check_status( mbus_get_tx_rx_pins( buffer ) );
    UART5_Write_Text( buffer );
    UART5_Write_Text( "\r\n" );

    // Set MBUS Mode
    UART5_Write_Text( "Setting MBUS Mode\r\n" );
    check_status( mbus_set_mbus_mode( ROLE_N1_METER ) );

    // Set RX / TX formats
    check_status( mbus_set_serial_tx_format( LENGTH_TX ) );   // TX Format must have Length.. so that sending / receiving works properly

    // Add a meter
    UART5_Write_Text( "Testing Address Setting...\r\n" );
    mbus_set_meter_address( new_meter );        //Test setting a meter
    UART5_Write_Text( "Successfully set the address\r\n" );
    mbus_get_meter_address( received_meter );   //Test getting a meter
    UART5_Write_Text( "Got the address\r\n" );
    UART5_Write_Text( long_text );
    UART5_Write_Text( "Now Adding Meter...\r\n" );
    status = mbus_add_edit_meter( DONT_FILTER, new_meter, 0, NO_ENCRYPT, NULL );
    if ( status == ERROR )
        UART5_Write_Text( "ERROR OCCURRED!\r\n" );
    else
        UART5_Write_Text( "Successful Meter Add!\r\n" );

    //Try sending some data...
    UART5_Write_Text( "\r\n\r\n Sending Frame Of Data...\r\n\r\n" );
    // Set OP Mode
    check_status( mbus_op_init() );
    //Send Frame
    sprintf( transmit_frame->data_field, "My Name is Corey Allen Lakey and I am %d Years Old.", 21 );
    transmit_frame->length_field = strlen( transmit_frame->data_field );
    UART5_Write_Text( transmit_frame->data_field );
    mbus_send_frame( transmit_frame, 1 );              /**< Using length field for TX format, so not length needed */

    while ( 1 );
}

```

